@@ -36,8 +36,12 @@ class FromEnvMachine(Machine):
 class TCPMachine(Machine):
     type: str = "tcp"
 
-    def __init__(self, name: str, host: str):
-        super(TCPMachine, self).__init__(name, host)
+    def __init__(self, name: str, host: str, port: Optional[Union[int, str]] = None):
+        configuration = {"host": host}
+        if port is not None:
+            configuration["port"] = port
+            host = f"{host}:{port}"
+        super(TCPMachine, self).__init__(name, host, configuration=configuration)
 
     @property
     def is_local(self) -> bool:
