@@ -209,7 +209,7 @@ class CPKFileMappingTrigger(Enum):
 class CPKFileMapping:
     source: str
     destination: str
-    triggers: List[CPKFileMappingTrigger]
+    triggers: List[str]
     required: bool
     mode: str = "rw"
 
@@ -230,7 +230,7 @@ class CPKFileMapping:
         return CPKFileMapping(
             source=data['source'],
             destination=data['destination'],
-            triggers=list(map(CPKFileMappingTrigger, data.get('triggers', ["default"]))),
+            triggers=data.get('triggers', ["default"]),
             required=data.get('required', False),
             mode=data.get('mode', "rw")
         )
@@ -369,6 +369,7 @@ class Machine(abc.ABC):
             endpoint_arch = client.info()["Architecture"]
             if endpoint_arch not in CANONICAL_ARCH:
                 raise CPKException(f"Unsupported architecture '{endpoint_arch}'.")
+            self._arch = endpoint_arch
         finally:
             client.close()
         return CANONICAL_ARCH[endpoint_arch]
@@ -386,12 +387,12 @@ class Machine(abc.ABC):
         if username is not None and password is not None:
             self.get_client().login(username=username, password=password)
 
-    def pull_image(self, image, progress=True) -> bool:
+    def pull_image(self, image, progress=True):
         client = self.get_client()
         layers = set()
         pulled = set()
         pbar = ProgressBar() if progress else None
-        for line in client.api.pull(image, stream=True, decode=True):
+        for line in client.api.pull(*image.split(":"), stream=True, decode=True):
             if "id" not in line or "status" not in line:
                 continue
             layer_id = line["id"]
@@ -404,7 +405,6 @@ class Machine(abc.ABC):
                 pbar.update(percentage)
         if progress:
             pbar.done()
-        return True
 
     def push_image(self, image, progress=True, **kwargs):
         client = self.get_client()
